/////////////////////////////////////////////////////////////////////////////////////
// File:	cgiinterface.h
// Author:	Walt Jones
// Email:	wjones@blarg.net
// Date:	6 August, 2000
// Purpose:	High quality API object to simplify CGI programming tasks
// Notes:	Much of the code contained in this cgiInterface object was originally part
//			of Thomas Boutell's CGIC library v1.0.  Boutell's original C code has been
//			converted into a C++ object, tweaked for usability and modified to include
//			STL support for strings and vectors.  Many of the private functions that
//			handle string parsing and the environment interface have been left virtually
//			as-is in their original C form.  Boutell's original CGIC library can be
//          downloaded from www.boutell.com/cgic
/////////////////////////////////////////////////////////////////////////////////////

// Make sure this file is included only once
#if !defined( CGIINTERFACE_H )
#define CGIINTERFACE_H

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include <vector>
#include <time.h>

// Redefine how string comparisons are done
#define cgiStrEq(a, b) (!strcmp((a), (b)))

// One form entry, consisting of an attribute-value pair.
typedef struct cgiFormEntryStruct {
        char *attr;
        char *value;
        struct cgiFormEntryStruct *next;
} cgiFormEntry;

// Return types definitions
typedef enum {
	cgiFileSuccess=1,
	cgiFileFail=0
} cgiFileResultType;

typedef enum {
	cgiFormSuccess=1,
	cgiFormNotFound=0,
	cgiFormBadType=-1
} cgiFormResultType;

typedef enum {
	cgiEscapeRest,
	cgiEscapeFirst,
	cgiEscapeSecond
} cgiEscapeState;

typedef enum {
	cgiUnescapeSuccess,
	cgiUnescapeMemory
} cgiUnescapeResultType;

typedef enum {
	cgiParseSuccess,
	cgiParseMemory,
	cgiParseIO
} cgiParseResultType;

typedef enum {
	cgiEnvironmentIO,
	cgiEnvironmentMemory,
	cgiEnvironmentSuccess
} cgiEnvironmentResultType;


class cgiInterface
{
private:
	char *contentLengthString;		// The content length as a string
	int hexValue[256];				// Hex values for characters
	char *findTarget;				// The find target
	cgiFormEntry *findPosition;		// The entry at the find position
	cgiFormEntry *firstFormEntry;	// The first form entry.
	
	// Environment variables
	char *serverSoftware;
	char *serverName;
	char *gatewayInterface;
	char *serverProtocol;
	char *serverPort;
	char *requestMethod;
	char *pathInfo;
	char *pathTranslated;
	char *scriptName;
	char *queryString;
	char *remoteHost;
	char *remoteAddr;
	char *authType;
	char *remoteUser;
	char *remoteIdentity;
	char *contentType;
	int contentLength;
	char *accept;
	char *userAgent;
	char *referrer;

	FILE *cgiIn;	// CGI input stream
	FILE *cgiOut;	// CGI output stream

	// Functions that parse form data
	cgiParseResultType parseFormGetInput() {return parseFormInput(queryString, contentLength);};
	cgiParseResultType parseFormPostInput();
	cgiParseResultType parseFormInput(char data[], const int length);
	cgiFormEntry *findFirstFormEntry(const char name[]);
	cgiFormEntry *findNextFormEntry();
	cgiFormResultType formEntryString(cgiFormEntry *e, string &result, const int newlines);
	
	// Other supporting functions
	void setupHEXConstants();
	void freeResources();
	void getEnv(char *s[], char var[]) {*s = getenv(var); if (!(*s)) {*s = "";} return;};
	bool cgiStrEqNc(const char s1[], const char s2[]);
	cgiUnescapeResultType unescapeChars(char *sp[], char cp[], const int length);
	int firstNonspaceChar(const char s[]);
	string replace(const string &original, const string &toFind, const string &toReplace);	

public:
	cgiInterface();
	~cgiInterface() {freeResources(); return;};

	// Functions that return environment variables
	string getServerSoftware() {return serverSoftware;};
	string getServerName() {return serverName;};
	string getGatewayInterface() {return gatewayInterface;};
	string getServerProtocol() {return serverProtocol;};
	string getServerPort() {return serverPort;};
	string getRequestMethod() {return requestMethod;};
	string getPathInfo() {return pathInfo;};
	string getPathTranslated() {return pathTranslated;};
	string getScriptName() {return scriptName;};
	string getQueryString() {return queryString;};
	string getRemoteHost() {return remoteHost;};
	string getRemoteAddr() {return remoteAddr;};
	string getAuthType() {return authType;};
	string getRemoteUser() {return remoteUser;};
	string getRemoteIdentity() {return remoteIdentity;};
	string getContentType() {return contentType;};
	int getContentLength() {return contentLength;};
	string getAccept() {return accept;};
	string getUserAgent() {return userAgent;}
	string getReferrer() {return referrer;};

	// Functions that return form entries
	cgiFormResultType getFormString(const char name[], string &result);
	cgiFormResultType getFormStringNoNewlines(const char name[], string &result);
	cgiFormResultType getFormSelectSingle(const char name[], string &result);
	cgiFormResultType getFormInteger(const char name[], int &result);
	cgiFormResultType getFormDouble(const char name[], double &result);
	cgiFormResultType getFormCheckboxSingle(const char name[]);
	cgiFormResultType getFormSelectMultiple(const char name[], vector<string> &result);
	cgiFormResultType getFormCheckboxMultiple(const char name[], vector<string> &result) {return getFormSelectMultiple(name, result);};
	cgiFormResultType getFormRadio(const char name[], string &result) {return getFormSelectSingle(name, result);};

	// Functions that print headers
	void printHeaderLocation(const char redirectUrl[]) {fprintf(cgiOut, "Location: %s%c%c", redirectUrl, 10, 10); return;};
	void printHeaderStatus(const int status, const char statusMessage[]) {fprintf(cgiOut, "Status: %d %s%c%c", status, statusMessage, 10, 10); return;};
	void printHeaderContentType(const char mimeType[]) {fprintf(cgiOut, "Content-type: %s%c%c", mimeType, 10, 10); return;};
	
	// Functions that print lines
	void printLn(const string &line, int num1, int num2) {fprintf(cgiOut, (line + "\n").c_str(), num1, num2); return;};
	void printLn(const string &line, double num1, double num2) {fprintf(cgiOut, (line + "\n").c_str(), num1, num2); return;};
	void printLn(const string &line, double num1, int num2) {fprintf(cgiOut, (line + "\n").c_str(), num1, num2); return;};
	void printLn(const string &line, int num1, double num2) {fprintf(cgiOut, (line + "\n").c_str(), num1, num2); return;};
	void printLn(const string &line, int num) {fprintf(cgiOut, (line + "\n").c_str(), num); return;};
	void printLn(const string &line, double num) {fprintf(cgiOut, (line + "\n").c_str(), num); return;};
	void printLn(const string &line) {fprintf(cgiOut, (line + "\n").c_str()); return;};
	cgiFileResultType printFile(const char filename[]);
	cgiFileResultType printFileHTML(const char filename[]);
	void printSelect(vector<string> &vect, const string &selected);
	void printSelect(vector<string> &val, vector<string> &vect, const string &selected);

	// Functions that free resources
	void freeStringArray(char *stringArray[]);
};

/////////////////////////////////////////////////////////////////////////////////////
// Method:	cgiInterface
// Purpose:	Constructor.  Gets environment variables and calls the correct function
//			to parse the input.
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
cgiInterface::cgiInterface()
{
	findTarget = 0;
	findPosition = 0;

	setupHEXConstants();

	getEnv(&serverSoftware, "SERVER_SOFTWARE");
	getEnv(&serverName, "SERVER_NAME");
	getEnv(&gatewayInterface, "GATEWAY_INTERFACE");
	getEnv(&serverProtocol, "SERVER_PROTOCOL");
	getEnv(&serverPort, "SERVER_PORT");
	getEnv(&requestMethod, "REQUEST_METHOD");
	getEnv(&pathInfo, "PATH_INFO");
	getEnv(&pathTranslated, "PATH_TRANSLATED");
	getEnv(&scriptName, "SCRIPT_NAME");
	getEnv(&queryString, "QUERY_STRING");
	getEnv(&remoteHost, "REMOTE_HOST");
	getEnv(&remoteAddr, "REMOTE_ADDR");
	getEnv(&authType, "AUTH_TYPE");
	getEnv(&remoteUser, "REMOTE_USER");
	getEnv(&remoteIdentity, "REMOTE_IDENT");
	getEnv(&contentType, "CONTENT_TYPE");
	getEnv(&contentLengthString, "CONTENT_LENGTH");
	contentLength = atoi(contentLengthString);	
	getEnv(&accept, "HTTP_ACCEPT");
	getEnv(&userAgent, "HTTP_USER_AGENT");
	getEnv(&referrer, "HTTP_REFERER");

	firstFormEntry = 0;
	cgiIn = stdin;
	cgiOut = stdout;

	// Parse the form input according to the request method
	if (cgiStrEqNc(requestMethod, "post"))
	{
		if (cgiStrEqNc(contentType, "application/x-www-form-urlencoded"))
			if (parseFormPostInput() != cgiParseSuccess)
				freeResources();
	}
	else if (cgiStrEqNc(requestMethod, "get"))
	{	
		// The spec says this should be taken care of by
		//	the server, but... it isn't
		contentLength = strlen(queryString);
		if (parseFormGetInput() != cgiParseSuccess)
			freeResources();
	}

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	parseFormPostInput
// Purpose:	Parses the input from a POST form
// Input:	None
// Output:	None
// Return:	cgiParseSuccess - The form input was parsed successfully
//			cgiParseMemory - A memory error occured
//			cgiParseIO - An IO error occured
/////////////////////////////////////////////////////////////////////////////////////
cgiParseResultType cgiInterface::parseFormPostInput()
{
	char *input;	// Input string
	cgiParseResultType result;	// Results of parsing the input string

	// If the form input is empty, there's nothing to do
	if (!contentLength)
		return cgiParseSuccess;

	// Get the input string
	input = (char *) malloc(contentLength);

	if (!input)
		return cgiParseMemory;	
	if (fread(input, 1, contentLength, cgiIn) != contentLength)
		return cgiParseIO;

	// Parse the input string
	result = parseFormInput(input, contentLength);

	free(input);
	return result;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	parseFormInput
// Purpose:	Parse form input to get the query string in the right format
// Input:	data - Input string from the form
//			length - Length of the input string
// Output:	None
// Return:	cgiParseSuccess - The form input was parsed successfully
//			cgiParseMemory - A memory error occured
//			cgiParseIO - An IO error occured
/////////////////////////////////////////////////////////////////////////////////////
cgiParseResultType cgiInterface::parseFormInput(char data[], const int length)
{
	int pos = 0;			// Current position
	cgiFormEntry *n;		// Temporary form entry
	cgiFormEntry *l = 0;	// Temproary form entry

	// Scan for pairs, unescaping and storing them as they are found.
	while (pos != length)
	{
		int foundEq = 0;
		int foundAmp = 0;
		int start = pos;
		int len = 0;
		char *attr;
		char *value;

		while (pos != length)
		{
			if (data[pos] == '=')
			{
				foundEq = 1;
				pos++;
				break;
			}
			pos++;
			len++;
		}
		if (!foundEq)
			break;

		if (unescapeChars(&attr, data+start, len) != cgiUnescapeSuccess)
			return cgiParseMemory;
	
		start = pos;
		len = 0;
		while (pos != length)
		{
			if (data[pos] == '&')
			{
				foundAmp = 1;
				pos++;
				break;
			}
			pos++;
			len++;
		}
		// The last pair probably won't be followed by a &, but
		//	that's fine, so check for that after accepting it
		if (unescapeChars(&value, data+start, len) != cgiUnescapeSuccess)
			return cgiParseMemory;
	
		// OK, we have a new pair, add it to the list.
		n = (cgiFormEntry *) malloc(sizeof(cgiFormEntry));	
		if (!n)
			return cgiParseMemory;

		// Save what we've found
		n->attr = attr;
		n->value = value;
		n->next = 0;

		if (!l)
			firstFormEntry = n;
		else
			l->next = n;

		l = n;
		if (!foundAmp)
			break;
	}
	return cgiParseSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	unescapeChars
// Purpose:	Clean escape characters from a string
// Input:	cp - string to clean
//			length - length of cp
// Output:	sp - string with escape characters removed
// Return:	cgiUnescapeSuccess - Unescape characters valid
//			cgiUnescapeMemory - A memory error occured
/////////////////////////////////////////////////////////////////////////////////////
cgiUnescapeResultType cgiInterface::unescapeChars(char *sp[], char cp[], const int length)
{
	char *s;	// A string
	cgiEscapeState escapeState = cgiEscapeRest;	// Escape state
	int escapedValue = 0;	// Escaped value
	int srcPos = 0;		// Source position
	int dstPos = 0;		// Destination position

	s = (char *) malloc(length + 1);
	if (!s)
		return cgiUnescapeMemory;

	// Parse the string and remove escape characters
	while (srcPos < length)
	{
		int ch = cp[srcPos];
		switch (escapeState)
		{
		case cgiEscapeRest:
			if (ch == '%')
				escapeState = cgiEscapeFirst;
			else if (ch == '+')
				s[dstPos++] = ' ';
			else
				s[dstPos++] = ch;	
			break;
		case cgiEscapeFirst:
			escapedValue = hexValue[ch] << 4;	
			escapeState = cgiEscapeSecond;
			break;
		case cgiEscapeSecond:
			escapedValue += hexValue[ch];
			s[dstPos++] = escapedValue;
			escapeState = cgiEscapeRest;
			break;
		}
		srcPos++;
	}
	s[dstPos] = '\0';
	*sp = s;
	return cgiUnescapeSuccess;
}		

/////////////////////////////////////////////////////////////////////////////////////
// Method:	setupHEXConstants
// Purpose:	Setup HEX constants in the hexValue array
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void cgiInterface::setupHEXConstants()
{
	for (int i=0; (i < 256); i++)
		hexValue[i] = 0;

	hexValue['0'] = 0;	
	hexValue['1'] = 1;	
	hexValue['2'] = 2;	
	hexValue['3'] = 3;	
	hexValue['4'] = 4;	
	hexValue['5'] = 5;	
	hexValue['6'] = 6;	
	hexValue['7'] = 7;	
	hexValue['8'] = 8;	
	hexValue['9'] = 9;
	hexValue['A'] = 10;
	hexValue['B'] = 11;
	hexValue['C'] = 12;
	hexValue['D'] = 13;
	hexValue['E'] = 14;
	hexValue['F'] = 15;
	hexValue['a'] = 10;
	hexValue['b'] = 11;
	hexValue['c'] = 12;
	hexValue['d'] = 13;
	hexValue['e'] = 14;
	hexValue['f'] = 15;

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	freeResources
// Purpose:	Frees CGI resources
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void cgiInterface::freeResources()
{
	cgiFormEntry *c = firstFormEntry;
	cgiFormEntry *n;
	while (c)
	{
		n = c->next;
		free(c->attr);
		free(c->value);
		free(c);
		c = n;
	}
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getFormString
// Purpose:	Gets the string value from the text field specified
// Input:	name - The text field to obtain a value from
// Output:	result - The string value from the text field specified
// Return:	cgiFormSuccess - The value was obtained successfully
//			cgiFormNotFound - The field specified was not found
/////////////////////////////////////////////////////////////////////////////////////
cgiFormResultType cgiInterface::getFormString(const char name[], string &result)
{
	cgiFormEntry *e;	// A form entry

	e = findFirstFormEntry(name);
	if (!e)
		return cgiFormNotFound;

	return formEntryString(e, result, 1);
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getFormStringNoNewlines
// Purpose:	Gets the string value from the text field specified and stomps all
//			newline characters
// Input:	name - The text field to obtain a value from
// Output:	result - The string value from the text field specified
// Return:	cgiFormSuccess - The value was obtained successfully
//			cgiFormNotFound - The field specified was not found
/////////////////////////////////////////////////////////////////////////////////////
cgiFormResultType cgiInterface::getFormStringNoNewlines(const char name[], string &result)
{
	cgiFormEntry *e;	// A form entry

	e = findFirstFormEntry(name);
	if (!e)
		return cgiFormNotFound;

	return formEntryString(e, result, 0);
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	formEntryString
// Purpose:	Gets the string value from the text field specified and stomps all new
//			line characters if desired
// Input:	e - The cgiFormEntry to extract the string value from
//			newlines - determines if newline characters are to be removed
// Output:	result - The string value from the text field specified
// Return:	cgiFormSuccess - The value was obtained successfully
/////////////////////////////////////////////////////////////////////////////////////
cgiFormResultType cgiInterface::formEntryString(cgiFormEntry *e, string &result, const int newlines)
{
	char *dp, *sp, *dpCopy;	// Strings
	int len = 0;			// Length
	int crCount = 0;		// CR count
	int lfCount = 0;		// LF count
	int ch;					// A character

	// Set up sp, dp and save a copy pointing to dp's start
	dp = new char[strlen(e->value)];
	dpCopy = dp;

	sp = e->value;
	
	while (true)
	{
		ch = *sp;
		
		// Fix the CR/LF, LF, CR nightmare: watch for
		//	consecutive bursts of CRs and LFs in whatever
		//	pattern, then actually output the larger number 
		//	of LFs. Consistently sane, yet it still allows
		//	consecutive blank lines when the user
		//	actually intends them.
		if ((ch == 13) || (ch == 10))
		{
			if (ch == 13)
				crCount++;
			else
				lfCount++;
		}
		else
		{
			if (crCount || lfCount)
			{
				int lfsAdd = crCount;
				if (lfCount > crCount)
					lfsAdd = lfCount;

				// Stomp all newlines if desired
				if (!newlines)
					lfsAdd = 0;

				while (lfsAdd)
				{
					*dp = 10;
					dp++;
					lfsAdd--;
					len++;		
				}
				crCount = 0;
				lfCount = 0;
			}
			if (ch == '\0') // The end of the source string
				break;	
	
			*dp = ch;
			dp++;
			len++;
		}
		sp++;	
	}
	
	dp--;
	ch = *dp;
	
	if(ch == 10)
		*dp = '\0';
	else
	{
		dp++;
		*dp = '\0';
	}
	if (!len)
	{
		result = "";
		return cgiFormSuccess;
	}

	// Copy our string into our result variable
	result = dpCopy;
	return cgiFormSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getFormInteger
// Purpose:	Gets the integer value from the text field specified
// Input:	name - The text field to obtain a value from
// Output:	result - The integer value from the text field specified
// Return:	cgiFormSuccess - The value was obtained successfully
//			cgiFormNotFound - The field specified was not found
//			cgiFormBadType - The value of the field specified was of an incorrect type
/////////////////////////////////////////////////////////////////////////////////////
cgiFormResultType cgiInterface::getFormInteger(const char name[], int &result)
 {
	cgiFormEntry *e;	// A form entry
	int ch;	// A character

	e = findFirstFormEntry(name);
	if (!e)
		return cgiFormNotFound; 

	if (!strlen(e->value))
	{
		result = NULL;
		return cgiFormSuccess;
	}

	ch = firstNonspaceChar(e->value);
	if (!(isdigit(ch)) && (ch != '-') && (ch != '+'))
		return cgiFormBadType;

	result = atoi(e->value);
	return cgiFormSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getFormDouble
// Purpose:	Gets the double value from the text field specified
// Input:	name - The text field to obtain a value from
// Output:	result - The double value from the text field specified
// Return:	cgiFormSuccess - The value was obtained successfully
//			cgiFormNotFound - The field specified was not found
//			cgiFormBadType - The value of the field specified was of an incorrect type
/////////////////////////////////////////////////////////////////////////////////////
cgiFormResultType cgiInterface::getFormDouble(const char name[], double &result)
{
	cgiFormEntry *e;	// A form entry
	int ch;	// A character

	e = findFirstFormEntry(name);
	if (!e)
		return cgiFormNotFound; 

	if (!strlen(e->value))
	{
		result = NULL;
		return cgiFormSuccess;
	}

	ch = firstNonspaceChar(e->value);
	if (!(isdigit(ch)) && (ch != '.') && (ch != '-') && (ch != '+'))
		return cgiFormBadType;

	result = atof(e->value);
	return cgiFormSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getFormSelectSingle
// Purpose:	Gets the value of the selected element from the single select list
//			specified 
// Input:	name - The single select list to obtain a value from
// Output:	result - The string value of the selected element
// Return:	cgiFormSuccess - The value was obtained successfully
//			cgiFormNotFound - The field specified was not found
/////////////////////////////////////////////////////////////////////////////////////
cgiFormResultType cgiInterface::getFormSelectSingle(const char name[], string &result) 
{
	cgiFormEntry *e;	// A form entry

	e = findFirstFormEntry(name);

	if (!e)
		return cgiFormNotFound;

	result = e->value;
	return cgiFormSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getFormSelectMultiple
// Purpose:	Gets the values of the selected elements from the multiple select list
//			specified
// Input:	name - The multiple select list to obtain a value from
// Output:	result - The string values of the selected elements
// Return:	cgiFormSuccess - The values were obtained successfully
//			cgiFormNotFound - The field specified was not found
/////////////////////////////////////////////////////////////////////////////////////
cgiFormResultType cgiInterface::getFormSelectMultiple(const char name[], vector<string> &result)
{
	cgiFormEntry *e;	// A form entry

	e = findFirstFormEntry(name);
	if (!e)
		return cgiFormNotFound;

	do
	{
		result.push_back(e->value);
	} while ((e = findNextFormEntry()) != 0);
	
	return cgiFormSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getFormCheckboxSingle
// Purpose:	Determines if the checkbox specified is checked
// Input:	name - The checkbox to obtain a value from
// Output:	None
// Return:	cgiFormSuccess - The checkbox was checked
//			cgiFormNotFound - The checkbox was not checked or did not exist
/////////////////////////////////////////////////////////////////////////////////////
cgiFormResultType cgiInterface::getFormCheckboxSingle(const char name[])
{
	cgiFormEntry *e;	// A form entry

	e = findFirstFormEntry(name);
	if (!e)
		return cgiFormNotFound;

	return cgiFormSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	cgiStrEqNc
// Purpose:	Determines if two strings are equal
// Input:	s1 - The first string
//			s2 - The second string
// Output:	None
// Return:	True - The two strings are equal
//			False - The two strings are not equal
/////////////////////////////////////////////////////////////////////////////////////
bool cgiInterface::cgiStrEqNc(const char s1[], const char s2[])
{
	char *ss1;	// Copy of s1
	char *ss2;	// Copy of s2

	// Set up ss1, ss2 and copy in the strings
	ss1 = new char[strlen(s1)];
	ss2 = new char[strlen(s2)];
	strcpy(ss1, s1);
	strcpy(ss2, s2);

	while(true)
	{
		if (!(*ss1))
		{
			if (!(*ss2))
				return true;
			else
				return false;
		}
		else if (!(*ss2))
			return false;

		if (isalpha(*ss1))
		{
			if (tolower(*ss1) != tolower(*ss2))
				return false;
		}
		else if ((*ss1) != (*ss2))
			return false;

		ss1++;
		ss2++;
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	findFirstFormEntry
// Purpose:	Finds the first entry in the form of the name specified
// Input:	name - The name of the entry to find
// Output:	None
// Return:	The first form entry of the name specified
/////////////////////////////////////////////////////////////////////////////////////
cgiFormEntry *cgiInterface::findFirstFormEntry(const char name[])
 {
 	findTarget = new char[strlen(name)];
	strcpy(findTarget, name);
	findPosition = firstFormEntry;
	return findNextFormEntry();
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	findNextFormEntry
// Purpose:	Finds the next entry in the form of the name specified
// Input:	None
// Output:	None
// Return:	The next form entry of the name specified
/////////////////////////////////////////////////////////////////////////////////////
cgiFormEntry *cgiInterface::findNextFormEntry()
 {
	while (findPosition)
	{
		cgiFormEntry *c = findPosition;
		findPosition = c->next;
		if (!strcmp(c -> attr, findTarget))
			return c;
	}
	return NULL;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	firstNonspaceChar
// Purpose:	Returns the index of the first non-space character in a string
// Input:	s - The string
// Output:	None
// Return:	The index of the first non-space character in s
/////////////////////////////////////////////////////////////////////////////////////
int cgiInterface::firstNonspaceChar(const char s[])
{
	int len = strspn(s, " \n\r\t");
	return s[len];
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	freeStringArray
// Purpose:	Frees the memory associated with a string
// Input:	stringArray - The string to free the memory of
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void cgiInterface::freeStringArray(char *stringArray[])
{
	char *p;
	p = *stringArray;
	while (p)
	{
		free(p);
		stringArray++;
		p = *stringArray;
	}

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	printFile
// Purpose:	Prints the file specified to the CGI output stream
// Input:	filename - The File to print
// Output:	None
// Return:	cgiFileSuccess - The file was printed successfully
//			cgiFileFail - The file could not be found
/////////////////////////////////////////////////////////////////////////////////////
cgiFileResultType cgiInterface::printFile(const char filename[])
{
	ifstream fin;	// Input file stream
	char templine[2048];	// Temporary line

	fin.open(filename);
	if (fin.fail())
		return cgiFileFail;

	while (!fin.eof())
	{
		fin.getline(templine, 2048, '\n');
		fprintf(cgiOut, "%s\n", templine);
	}
	fin.close();

	return cgiFileSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	printFileHTML
// Purpose:	Prints the file specified in HTML format to the CGI output stream
// Input:	filename - The File to print
// Output:	None
// Return:	cgiFileSuccess - The file was printed successfully
//		cgiFileFail - The file could not be found
/////////////////////////////////////////////////////////////////////////////////////
cgiFileResultType cgiInterface::printFileHTML(const char filename[])
{
	ifstream fin;	// Input file stream
	char templine[2048];	// Temporary line
	string tempStr;	// Temporary string

	fin.open(filename);
	if (fin.fail())
		return cgiFileFail;

	while (!fin.eof())
	{
		fin.getline(templine, 2048, '\n');
		tempStr = string(templine);
		tempStr = replace(tempStr, "<", "&lt;");
		printLn(tempStr + "<BR>");
	}
	fin.close();

	return cgiFileSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	printSelect
// Purpose:	Prints the vector specified as options in an html select element to the
//		CGI output stream
// Input:	vect - The vector to print
//		selected - The element to mark as selected.  Default is NULL
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void cgiInterface::printSelect(vector<string> &vect, const string &selected = "")
{
	vector<string>::iterator end, it;

	// Go through and print out the vector
	end = vect.end();
	if (selected.length() > 0)
		for (it=vect.begin(); it != end; it++)
		{
			if (*it == selected)
				printLn("<OPTION VALUE=\"" + *it + "\" SELECTED>" + *it);
			else
				printLn("<OPTION VALUE=\"" + *it + "\">" + *it);
		}
	else
		for (it=vect.begin(); it != end; it++)
			printLn("<OPTION VALUE=\"" + *it + "\">" + *it);
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	printSelect
// Purpose:	Prints the vector specified as options in an html select element to the
//		CGI output stream
// Input:	val - Values to use for the select element
//		vect - The vector to print
//		selected - The element to mark as selected.  Default is NULL
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void cgiInterface::printSelect(vector<string> &val, vector<string> &vect, const string &selected = "")
{
	vector<string>::iterator end, it, itV;

	// Go through and print out the vector
	end = vect.end();
	if (selected.length() > 0)
		for (it=vect.begin(), itV=val.begin(); it != end; it++, itV++)
		{
			if (*it == selected)
				printLn("<OPTION VALUE=\"" + *itV + "\" SELECTED>" + *it);
			else
				printLn("<OPTION VALUE=\"" + *itV + "\">" + *it);
		}
	else
		for (it=vect.begin(), itV=val.begin(); it != end; it++, itV++)
			printLn("<OPTION VALUE=\"" + *itV + "\" SELECTED>" + *it);
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	replace
// Purpose:	Replaces the specified substring with new text
// Input:	original - The original string
//		toFind - The substring to replace
//		toReplace - The text that will replace toFind
// Output:	None
// Return:	The string with the substring replaced
/////////////////////////////////////////////////////////////////////////////////////
string cgiInterface::replace(const string &original, const string &toFind, const string &toReplace)
{
	string newStr = original;
	int i = newStr.find(toFind, 0);
	
	while (i != string::npos)
	{
		newStr = newStr.erase(i, toFind.length());
		newStr = newStr.insert(i, toReplace);
		i = newStr.find(toFind, i);
	}
	
	return newStr;
}

#endif // defined( CGIINTERFACE_H )
