/////////////////////////////////////////////////////////////////////////////////////
// File:	errorlog.h
// Author:	Walt Jones
// Email:	wjones@blarg.net
// Date:	2 July, 2000
// Project:	ResNet Autoreg v2.0
// Purpose:	System error log object
/////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include <vector>
#include <time.h>
#include "timeutils.h"
#include "filelock.h"

// Return types definitions
typedef enum {
	errorLogSuccess=1,
	errorLogFail=0
} errorLogResultType;

class errorLogInterface
{
private:
	// Variables
	ifstream fin;		// Input filestream
	ofstream fout;		// Outfile filestream
	char *filename;		// Filename
	fileLock lock;		// File lock

	// Static class variables
	static const int maxStringSize;

public:
	// Methods
	errorLogInterface(const string &file);
	~errorLogInterface();
	errorLogResultType clear();
	errorLogResultType addEntry(const string &entry);
	errorLogResultType getContents(vector<string> &contents);
};

// Initialize static class variables
const int errorLogInterface::maxStringSize = 256;

/////////////////////////////////////////////////////////////////////////////////////
// Method:	errorLogInterface
// Purpose:	Constructor
// Input:	file - The filename (with full path) to the log file to use
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
errorLogInterface::errorLogInterface(const string &file)
{
	// Set up arrays
	filename = new char[maxStringSize + 1];

	// Set up the filename
	strcpy(filename, file.c_str());

	// Set up the fileLock
	lock.setFilename(filename);

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	~errorLogInterface
// Purpose:	Destructor
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
errorLogInterface::~errorLogInterface()
{
	// Close files
	if (lock.isLocked())
		lock.release();
	fin.close();
	fout.close();
	delete filename;
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	clear
// Purpose:	Clears the file
// Input:	None
// Output:	None
// Return:	errorLogSuccess - The log file was cleared successfully
//			errorLogFail - An error occured when attempting to open the log file
/////////////////////////////////////////////////////////////////////////////////////
errorLogResultType errorLogInterface::clear()
{
	// Open the file to clear it
	lock.lock();
	fout.open(filename, ios::trunc);
	if (fout.fail())
	{
		lock.release();
		return errorLogFail;
	}

	fout.close();
	lock.release();

	return errorLogSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	addEntry
// Purpose:	Appends a log entry to the file
// Input:	entry - The entry to add
// Output:	None
// Return:	errorLogSuccess - The entry was appended successfully
//			errorLogFail - An error occured when attempting to open the log file
/////////////////////////////////////////////////////////////////////////////////////
errorLogResultType errorLogInterface::addEntry(const string &entry)
{
	// Get the current time;
	string time = getCurrTime();
	
	// Open the file
	lock.lock();
	fout.open(filename, ios::app);
	if (fout.fail())
	{
		lock.release();
		return errorLogFail;
	}

	// Write the entry
	fout << "[" << time.c_str() << "] " << entry.c_str() << endl;

	// Close the file
	fout.close();
	lock.release();
	return errorLogSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getContents
// Purpose:	Obtains the contents of the log file and places them in a vector
// Input:	contents - The vector containing the contents of the logfile
// Output:	None
// Return:	errorLogSuccess - The contents were obtained successfully
//			errorLogFail - An error occured when attempting to open the log file
/////////////////////////////////////////////////////////////////////////////////////
errorLogResultType errorLogInterface::getContents(vector<string> &contents)
{
	char templine[maxStringSize + 1];		// Temporary line

	// Open the file
	lock.lock();
	fin.open(filename);
	if(fin.fail())
	{
		lock.release();
		return errorLogFail;
	}

	// Make sure the vector is empty
	if (!contents.empty())
		contents.clear();

	// Read the contents of the file and dump it into the vector
	while(!fin.eof())
	{
		fin.getline(templine, maxStringSize, '\n');
		contents.push_back(string(templine));
	}

	// Close the file
	fin.close();
	lock.release();
	return errorLogSuccess;
}